package com.mortals.xhx.module.record.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.data.PictureRenderData;
import com.deepoove.poi.data.PictureType;
import com.deepoove.poi.data.Pictures;
import com.deepoove.poi.plugin.table.MultipleRowTableRenderPolicy;
import com.deepoove.poi.util.RegexUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.base.system.user.model.UserEntity;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.common.utils.*;
import com.mortals.xhx.module.certificate.model.*;
import com.mortals.xhx.module.certificate.model.vo.CertificateDocumentPdu;
import com.mortals.xhx.module.certificate.pdu.ApplyLogPdu;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import com.mortals.xhx.module.certificate.service.CertificateDocumentService;
import com.mortals.xhx.module.child.model.ChildLicenseEntity;
import com.mortals.xhx.module.child.model.ChildLicenseQuery;
import com.mortals.xhx.module.child.service.ChildLicenseService;
import com.mortals.xhx.module.record.dao.RecordDao;
import com.mortals.xhx.module.record.model.*;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.http.entity.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.record.dao.ApplyLogDao;
import com.mortals.xhx.module.record.service.ApplyLogService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
* ApplyLogService
* 证照申请 service实现
*
* @author zxfei
* @date 2022-10-14
*/
@Service("applyLogService")
public class ApplyLogServiceImpl extends AbstractCRUDServiceImpl<ApplyLogDao, ApplyLogEntity, Long> implements ApplyLogService {

    @Value("${upload.path}")
    private String filePath;

    /** 应用公网地址 */
    @Value("${serviceUrl:http://one.scsmile.cn/h5/#?id=}")
    private String serviceUrl;

    @Autowired
    private UploadService uploadService;

    @Autowired
    private CertificateCatalogService certificateCatalogService;

    @Autowired
    private PrintWaitQueueService printWaitQueueService;

    @Autowired
    private RecordDao recordDao;

    @Autowired
    private UserService userService;
    @Autowired
    private ChildLicenseService childLicenseService;

    @Override
    protected void removeAfter(Long[] ids, Context context,int result) throws AppException {
        for (Long id:ids){
            PrintWaitQueueEntity condition = new PrintWaitQueueEntity();
            condition.setApplyId(id);
            printWaitQueueService.remove(condition,context);
        }
    }

    @Override
    public ApplyLogEntity get(Long key, Context context) throws AppException {
        ApplyLogEntity entity = this.dao.get(key);
        if(entity!=null){
            List<ChildLicenseEntity> childs = childLicenseService.find(new ChildLicenseQuery().applyId(key).processStatus(ProcessStatusEnum.已处理.getValue()).childStatus(StatusEnum.ENABLE.getValue()));
            entity.setChildLicense(childs);
        }
        return entity;
    }

    @Override
    protected ApplyLogEntity findBefore(ApplyLogEntity entity, PageInfo pageInfo, Context context) throws AppException {
        ApplyLogQuery query = new ApplyLogQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            ApplyLogQuery condition1 = new ApplyLogQuery();
            ApplyLogQuery condition2 = new ApplyLogQuery();
            condition1.setHolderName(condition.toString());
            condition2.setCertificateCode(condition.toString());
            List<ApplyLogQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            query.setOrConditionList(orConditionList);
        }
        query.setRecordStatus(YesNoEnum.NO.getValue());        //已归档的不显示
        return query;
    }

    @Override
    protected void findAfter(ApplyLogEntity params, PageInfo pageInfo, Context context, List<ApplyLogEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            for(ApplyLogEntity entity:list){
                List<ChildLicenseEntity> childLicenseList = childLicenseService.find(new ChildLicenseQuery().applyId(entity.getId()));
//                if(CollectionUtils.isEmpty(childLicenseList)){
//                    entity.setProcessStatus("0/0");
//                }else {
//                    Map<Integer, List<ChildLicenseEntity>> group = childLicenseList.stream().collect(Collectors.groupingBy(x -> x.getProcessStatus()));
//                    int pCount = CollectionUtils.isEmpty(group.get(1)) ? 0 : group.get(1).size();
//                    entity.setProcessStatus(pCount + "/" + childLicenseList.size());
//                }
                int childSum = 0;
                int pCount = 0;
                List<CertificateDocumentPdu> documentEntityList = new ArrayList<>();
                for(ChildLicenseEntity item:childLicenseList){
                    CertificateDocumentEntity documentEntity = new CertificateDocumentEntity();
                    BeanUtils.copyProperties(item,documentEntity);
                    documentEntity.setId(item.getDocumentId());
                    documentEntity.setStatus(StatusEnum.DISABLE.getValue());
                    if(item.getChildStatus()==StatusEnum.ENABLE.getValue()){
                        documentEntity.setStatus(StatusEnum.ENABLE.getValue());
                        childSum++;
                        if(item.getProcessStatus()==ProcessStatusEnum.已处理.getValue().intValue()){
                            pCount++;
                        }
                    }
                    CertificateDocumentPdu pdu = new CertificateDocumentPdu();
                    BeanUtils.copyProperties(documentEntity, pdu);
                    documentEntityList.add(pdu);
                }
                entity.setProcessStatus(pCount + "/" + childSum);
                entity.setChildCertificate(documentEntityList);
            }
        }
    }

    @Override
    protected void validData(ApplyLogEntity entity, Context context) throws AppException {
//        if(StringUtils.isEmpty(entity.getCertificateCode())){
//            throw new AppException("许可证编号不能为空");
//        }
//        if(entity.getIssueTime()==null){
//            throw new AppException("制证时间不能为空");
//        }
//        if(StringUtils.isEmpty(entity.getLegalPerson())){
//            throw new AppException("法定代表人不能为空");
//        }
//        if(StringUtils.isEmpty(entity.getBusinessPlace())){
//            throw new AppException("经营场所不能为空");
//        }
//        if(StringUtils.isEmpty(entity.getAuthority())){
//            throw new AppException("制证机关不能为空");
//        }
        if(StringUtils.isEmpty(entity.getSocialCode())){
            entity.setSocialCode(DateUtils.getCurrDateTime("yyyyMMddHHmmss")+RandomUtil.randomNumbers(4));
            //throw new AppException("统一社会信用代码不能为空");
        }
//        if(StringUtils.isEmpty(entity.getLicenseProject())){
//            throw new AppException("许可项目不能为空");
//        }
        if(StringUtils.isEmpty(entity.getEnterpriseName())){
            throw new AppException("市场主体名称不能为空");
        }
//        if(StringUtils.isEmpty(entity.getIndustryName())){
//            throw new AppException("行业名称不能为空");
//        }
        if(entity.getCatalogId()==null){
            throw new AppException("目录ID不能为空");
        }
        super.validData(entity, context);
    }

    private void createFormContent(ApplyLogEntity applyLogEntity){
        JSONObject formContentJson = new JSONObject();
        formContentJson.put("i_1_市场主体名称",applyLogEntity.getEnterpriseName());
//        formContentJson.put("i_2_许可证编号",applyLogEntity.getCertificateCode());
//        formContentJson.put("i_3_统一社会信用代码",applyLogEntity.getSocialCode());
//        formContentJson.put(" i_5_法定代表人（负责人）",applyLogEntity.getLegalPerson());
//        formContentJson.put("i_8_经营场所",applyLogEntity.getBusinessPlace());
//        formContentJson.put("i_9_行业类别",applyLogEntity.getIndustryName());
//        formContentJson.put("i_10_许可项目",applyLogEntity.getLicenseProject());
//        formContentJson.put("i_11_制证日期", DateUtils.getStrDate(applyLogEntity.getIssueTime()));
//        formContentJson.put("i_14_制证机关",applyLogEntity.getAuthority());
        formContentJson.put("@qrcode_1_二维码",applyLogEntity.getSocialCode());
        if(CollectionUtils.isNotEmpty(applyLogEntity.getChildCertificate())){
            for(int i = 0;i<applyLogEntity.getChildCertificate().size();i++){
                int index = i+1;
                formContentJson.put("i_"+ index +"_子证",applyLogEntity.getChildCertificate().get(i).getShortName());
            }
        }
        applyLogEntity.setFormContent(formContentJson.toJSONString());

    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ApplyLogEntity save(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        validData(applyLogEntity,context);
        createFormContent(applyLogEntity);
        Long oldId = null;
        boolean is_record = false;
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        applyLogEntity.setCatalogName(catalog.getCatalogName());
        applyLogEntity.setFormTemplate(catalog.getFormContent());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getTemplateFileUrl(),applyLogEntity.getFormContent());
        try {
            String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
            String filePath = rootPath + "certificate/qrcode/";
            String qrcodeName =  applyLogEntity.getCertificateCode() + ".png";
            File pathDir = new File(filePath);
            if (!pathDir.exists()) {
                pathDir.mkdirs();
            }
            String imagepath = filePath + qrcodeName;
            String qrCode = serviceUrl+applyLogEntity.getSocialCode();
            QrCodeUtil.generateQrCodeFile(qrCode,imagepath);
            applyLogEntity.setQRCode("/certificate/qrcode/"+qrcodeName);
            //entry.setValue(pictureRenderData);
        } catch (Exception e) {
            log.error("error", e);
        }
        docTemplate.setQrCode(applyLogEntity.getQRCode());
        boolean hasPDF = false;
        if(catalog.getIsPdf()!=null && catalog.getIsPdf()==1){
            hasPDF = true;
        }
        String paths = preview(docTemplate, context,hasPDF);
        String[] vals = paths.split(";");
        if(vals.length==3) {
            //有pdf时
            applyLogEntity.setCertificateUrl(vals[2]);
            applyLogEntity.setPreviewUrl(vals[1]);
        }
        if(vals.length==2) {
            //无pdf时
            applyLogEntity.setCertificateUrl(vals[0]);
            applyLogEntity.setPreviewUrl(vals[1]);
        }
        applyLogEntity.setCreateTime(new Date());
        applyLogEntity.setRecordStatus(YesNoEnum.NO.getValue());
        if(context!=null && context.getUser()!=null) {
            applyLogEntity.setCreateUserId(context.getUser().getId());
        }

        applyLogEntity.setGenerateStatus(GenerateStatus.ORIGINAL.getValue());
        if(applyLogEntity.getId()!=null && applyLogEntity.getRecordId()!=null){
            applyLogEntity.setOperType(OperTypeEnum.UPDATE.getValue());
            oldId = applyLogEntity.getId();
            is_record = true;
            RecordEntity recordEntity = new RecordEntity();
            recordEntity.setId(applyLogEntity.getRecordId());
            recordEntity.setUpdateUserId(applyLogEntity.getCreateUserId());
            recordEntity.setUpdateTime(applyLogEntity.getCreateTime());
            recordDao.update(recordEntity);
        }else {
            applyLogEntity.setOperType(OperTypeEnum.SAVE.getValue());
            RecordEntity recordEntity = new RecordEntity();
            recordEntity.setCatalogId(catalog.getId());
            recordEntity.setCatalogCode(catalog.getCatalogCode());
            recordEntity.setCatalogName(catalog.getCatalogName());
            recordEntity.setRecordStatus(YesNoEnum.YES.getValue());
            recordEntity.setCreateUserId(applyLogEntity.getCreateUserId());
            recordEntity.setCreateTime(applyLogEntity.getCreateTime());
            recordDao.insert(recordEntity);
            applyLogEntity.setRecordId(recordEntity.getId());
        }
        dao.insert(applyLogEntity);
        if(is_record){
            ApplyLogEntity updata = new ApplyLogEntity();
            updata.setId(oldId);
            updata.setRecordStatus(YesNoEnum.YES.getValue());
            updata.setUpdateTime(new Date());
            dao.update(updata);
            PrintWaitQueueEntity condition = new PrintWaitQueueEntity();
            condition.setApplyId(oldId);
            printWaitQueueService.remove(condition,context);
        }
        //插入子证联报
        childLicenseService.createChildLicense(applyLogEntity,oldId,context);
        printWaitQueueService.creatWaitQueueByApply(applyLogEntity, GenerateStatus.ORIGINAL.getValue(),context);
        applyLogEntity.setFormContent(null);
        applyLogEntity.setFormTemplate(null);
        return applyLogEntity;
    }

    @Override
    public String previewOriginal(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        validData(applyLogEntity,context);
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getTemplateFileUrl(),applyLogEntity.getFormContent());
        return preview(docTemplate, context,false);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void generateOriginal(ApplyLogEntity applyLogEntity, Context context) throws AppException {
        validData(applyLogEntity,context);
        Long oldId = null;
        boolean is_record = false;
        CertificateCatalogEntity catalog = certificateCatalogService.get(applyLogEntity.getCatalogId());
        applyLogEntity.setCatalogCode(catalog.getCatalogCode());
        applyLogEntity.setCatalogName(catalog.getCatalogName());
        applyLogEntity.setFormTemplate(catalog.getFormContent());
        DocTemplateVO docTemplate = new DocTemplateVO(catalog.getTemplateFileUrl(),applyLogEntity.getFormContent());
        boolean hasPDF = false;
        if(catalog.getIsPdf()!=null && catalog.getIsPdf()==1){
            hasPDF = true;
        }
        String paths = preview(docTemplate, context,hasPDF);
        String[] vals = paths.split(";");
        if(vals.length==3) {
            //有pdf时
            applyLogEntity.setCertificateUrl(vals[2]);
            applyLogEntity.setPreviewUrl(vals[1]);
        }
        if(vals.length==2) {
            //无pdf时
            applyLogEntity.setCertificateUrl(vals[0]);
            applyLogEntity.setPreviewUrl(vals[1]);
        }
        applyLogEntity.setCreateTime(new Date());
        applyLogEntity.setRecordStatus(YesNoEnum.NO.getValue());
        if(context!=null && context.getUser()!=null) {
            applyLogEntity.setCreateUserId(context.getUser().getId());
        }
        try {
            String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
            String filePath = rootPath + "certificate/qrcode/";
            String qrcodeName =  applyLogEntity.getCertificateCode() + ".png";
            File pathDir = new File(filePath);
            if (!pathDir.exists()) {
                pathDir.mkdirs();
            }
            String imagepath = filePath + qrcodeName;
            String qrCode = serviceUrl+applyLogEntity.getSocialCode();
            QrCodeUtil.generateQrCodeFile(qrCode,imagepath);
            applyLogEntity.setQRCode("/certificate/qrcode/"+qrcodeName);
            //entry.setValue(pictureRenderData);
        } catch (Exception e) {
            log.error("error", e);
        }

        applyLogEntity.setGenerateStatus(GenerateStatus.ORIGINAL.getValue());
        if(applyLogEntity.getId()!=null && applyLogEntity.getRecordId()!=null){
            applyLogEntity.setOperType(OperTypeEnum.UPDATE.getValue());
            oldId = applyLogEntity.getId();
            is_record = true;
            RecordEntity recordEntity = new RecordEntity();
            recordEntity.setId(applyLogEntity.getRecordId());
            recordEntity.setUpdateUserId(applyLogEntity.getCreateUserId());
            recordEntity.setUpdateTime(applyLogEntity.getCreateTime());
            recordDao.update(recordEntity);
        }else {
            applyLogEntity.setOperType(OperTypeEnum.SAVE.getValue());
            RecordEntity recordEntity = new RecordEntity();
            recordEntity.setCatalogId(catalog.getId());
            recordEntity.setCatalogCode(catalog.getCatalogCode());
            recordEntity.setCatalogName(catalog.getCatalogName());
            recordEntity.setRecordStatus(YesNoEnum.YES.getValue());
            recordEntity.setCreateUserId(applyLogEntity.getCreateUserId());
            recordEntity.setCreateTime(applyLogEntity.getCreateTime());
            recordDao.insert(recordEntity);
            applyLogEntity.setRecordId(recordEntity.getId());
        }
        dao.insert(applyLogEntity);
        if(is_record){
            ApplyLogEntity updata = new ApplyLogEntity();
            updata.setId(oldId);
            updata.setRecordStatus(YesNoEnum.YES.getValue());
            updata.setUpdateTime(new Date());
            dao.update(updata);
            PrintWaitQueueEntity condition = new PrintWaitQueueEntity();
            condition.setApplyId(oldId);
            printWaitQueueService.remove(condition,context);
        }
        //插入子证联报
        childLicenseService.createChildLicense(applyLogEntity,oldId,context);
        printWaitQueueService.creatWaitQueueByApply(applyLogEntity, GenerateStatus.ORIGINAL.getValue(),context);
    }

    private String preview(DocTemplateVO docTemplate,Context context,boolean hasPDF){
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        //转换表单参数为map集合
        ObjectMapper mapper = new ObjectMapper();
        ConfigureBuilder builder = Configure.builder();
        builder.buildGrammerRegex(RegexUtils.createGeneral("{{", "}}"));

        try {
            Map<String, Object> data = mapper.readValue(docTemplate.getFormContent(), Map.class);
            Map<String, Object> addMap = new HashMap<>();
            //遍历查看是否有图片数据，多选框
            data.entrySet().stream().forEach(entry -> {
                if (entry.getKey().indexOf("@image") != -1) {
                    //获取值 读取本地图片
                    String imagepath = uploadService.getFilePath(entry.getValue().toString());
                    log.info("Image path: " + imagepath);
                    try {
                        PictureRenderData pictureRenderData = Pictures.ofStream(new FileInputStream(imagepath), PictureType.JPEG)
                                .size(100, 120).create();
                        addMap.put(StrUtil.removePrefixIgnoreCase(entry.getKey(),"@"),pictureRenderData);
                        //entry.setValue(pictureRenderData);
                    } catch (FileNotFoundException e) {
                        log.error("error", e);
                    }
                }
                //构建填充复选框值
                if (entry.getKey().indexOf("ck_") != -1 && entry.getKey().indexOf("<") != -1) {
                    //获取复选框选项，并渲染值
                    String ckStr = com.mortals.framework.util.StringUtils.substringBetween(entry.getKey(), "<", ">");
                    //获取所有选项
                    List<String> checkboxs = StrUtil.splitTrim(ckStr, "-", -1);
                    //选中的项
                    List<String> ckList = Convert.toList(String.class, entry.getValue());
                    //合并
                    String ckStrs = checkboxs.stream().map(item -> {
                        for (String checked : ckList) {
                            if (checked.equals(item)) {
                                Map<Boolean, String> map = new HashMap<>();
                                map.put(true, item);
                                return map;
                            }
                        }
                        Map<Boolean, String> map = new HashMap<>();
                        map.put(false, item);
                        return map;
                    }).map(m -> m.entrySet().stream().map(m1 -> ExportDocUtil.createCheckBoxStrOther(m1.getKey(), m1.getValue())).findFirst().orElseGet(() -> "")
                    ).collect(Collectors.joining(""));
                    entry.setValue(ckStrs);
                }
                //TODO 单选
                //绑定动态表单
                if (entry.getKey().indexOf("dt_") != -1) {
                    builder.bind(entry.getKey(), new MultipleRowTableRenderPolicy());
                }
                //二维码输入框处理
                if (entry.getKey().indexOf("@qrcode") != -1) {
                    try {
//                        String filePath = rootPath+"file/qrcode/";
//                        String qrcodeName =  new Date().getTime() + ".png";
//                        File pathDir = new File(filePath);
//                        if (!pathDir.exists()) {
//                            pathDir.mkdirs();
//                        }
//                        String imagepath = filePath+qrcodeName;
//                        QrCodeUtil.generateQrCodeFile(entry.getValue().toString(),imagepath);
                        String imagepath = rootPath + docTemplate.getQrCode();
                        PictureRenderData pictureRenderData = Pictures.ofStream(new FileInputStream(imagepath), PictureType.PNG)
                                .size(150, 150).create();
                        addMap.put(StrUtil.removePrefixIgnoreCase(entry.getKey(),"@"),pictureRenderData);
                        //entry.setValue(pictureRenderData);
                    } catch (FileNotFoundException e) {
                        log.error("error", e);
                    }
                }
            });
            if(addMap.size()>0){
                data.putAll(addMap);
            }
            //查询是否有多选框，
            //获取模板文件地址
            String path = rootPath + docTemplate.getTemplatePath();
            String templateName = FileUtil.getName(path);
            Configure config = builder.build();
            XWPFTemplate template = null;
            ByteArrayOutputStream byteArrayOutputStream = null;
            try {
                template = XWPFTemplate.compile(path, config).render(data);
                byteArrayOutputStream = new ByteArrayOutputStream();
                template.write(byteArrayOutputStream);

                MultipartFile multipartFile = new MockMultipartFile(templateName, templateName,
                        ContentType.APPLICATION_OCTET_STREAM.toString(), byteArrayOutputStream.toByteArray());
                String mergedocPath = uploadService.saveFileUpload(multipartFile, "/mergedoc", context==null?null:context.getUser());
                log.info("mergedocPath:" + mergedocPath);
                String mergedoc = rootPath + mergedocPath;
                //转换预览图片
                String fileName = RandomUtil.randomNumbers(12) + ".jpg";
                String preView = this.filePath + "/preview/" + fileName;
                File preViewPathDir = new File(this.filePath + "/preview/");
                if (!preViewPathDir.exists()) {
                    preViewPathDir.mkdirs();
                }
                WordUtil.convertWordToJPEG(mergedoc, preView);
                log.info("preView:" + "/preview/" + fileName);
                if(hasPDF) {
                    String pdfName = mergedocPath.substring(0,mergedocPath.indexOf(".")) + ".pdf";
                    String pdfPath = rootPath + pdfName;
                    WordUtil.convertWordToPDF(mergedoc, pdfPath);
                    //PdfUtil.docxToPdf(mergedoc, pdfPath);
                    //下载地址拼装
                    String returnStr = mergedocPath + ";" + "/preview/" + fileName + ";" + pdfName;
                    return returnStr;
                }else {
                    //下载地址拼装
                    String returnStr = mergedocPath + ";" + "/preview/" + fileName;
                    return returnStr;
                }
            } catch (Exception e) {
                log.error("渲染模板失败：", e);
            } finally {
                try {
                    template.close();
                    byteArrayOutputStream.close();
                } catch (IOException e) {
                    log.error("ioEx：", e);
                }
            }
        } catch (JsonProcessingException e) {
            log.error("转换异常", e);
            throw new AppException("表单参数转换异常:" + e.getMessage());
        }
        return "";
    }

    @Override
    public List<ApplyLogEntity> ApplyRecord(Long applyId) throws AppException {
        ApplyLogEntity applyLogEntity = this.dao.get(applyId);
        if(applyLogEntity==null){
            throw new AppException("数据不存在");
        }
        UserEntity userEntity = userService.get(applyLogEntity.getCreateUserId());
        ApplyLogEntity query = new ApplyLogEntity();
        query.setRecordId(applyLogEntity.getRecordId());
        List<ApplyLogEntity> list = dao.getList(query);
        for(ApplyLogEntity entity:list){
            OperTypeEnum operTypeEnum = OperTypeEnum.getByValue(entity.getOperType());
            if(operTypeEnum!=null){
                entity.setOperTypeName(operTypeEnum.getDesc()+entity.getCertificateName());
            }

            if(userEntity!=null){
                entity.setCreateUserName(userEntity.getRealName());
            }
        }
        return list;
    }

    @Override
    public String importData(List<Map<Integer, Object>> dataList, Map<String,String> fileMap, Long catalogId, Long templateId, Context context) throws AppException {
        if(catalogId==null){
            throw new AppException("目录ID不能为空");
        }
        CertificateCatalogEntity catalog = certificateCatalogService.get(catalogId);
        if(catalog==null){
            throw new AppException("目录ID不正确");
        }
        if(templateId==null){
            throw new AppException("目录模板ID不能为空");
        }

        JSONObject formContent = JSONObject.parseObject(catalog.getFormContent());
        JSONArray formList = formContent.getJSONArray("list");
        Map<Integer, Object> header = dataList.get(0); //表头
        List<ApplyLogEntity> applyLogList = new ArrayList<>();
        for (int r=1;r<dataList.size();r++){
            Map<Integer, Object> row = dataList.get(r);
            ApplyLogEntity entity = new ApplyLogEntity();
            entity.setCatalogId(catalogId);
            entity.setCatalogCode(catalog.getCatalogCode());
            entity.setCatalogName(catalog.getCatalogName());
            entity.setCertificateCode(String.valueOf(row.get(0)));
            entity.setCertificateName(String.valueOf(row.get(1)));
            entity.setHolderName(String.valueOf(row.get(2)));
            HolderType holderType = HolderType.getByDesc(String.valueOf(row.get(3)));
            entity.setHolderType(holderType==null?HolderType.LEGAL_AND_PERSON.getValue():holderType.getValue());
            HolderIdType holderIdType = HolderIdType.getByDesc(String.valueOf(row.get(4)));
            entity.setHolderIdType(holderIdType==null?HolderIdType.ID_CARD.getValue():holderIdType.getValue());
            entity.setHolderIDCardNo(String.valueOf(row.get(5)));
            entity.setPickerName(String.valueOf(row.get(6)));
            entity.setPickerIDCardNo(String.valueOf(row.get(7)));

            try {
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
                entity.setIssueTime(formatter.parse(String.valueOf(row.get(8))));
            }catch (ParseException e){
                entity.setIssueTime(new Date());
            }
            entity.setPrivateID(String.valueOf(row.get(9)));
            entity.setEnterpriseName(String.valueOf(row.get(10)));
            JSONObject formJson = new JSONObject();
            int formSize = row.size();
            for(int i = 11;i<formSize;i++) {
                String key = String.valueOf(header.get(i));
                String value = String.valueOf(row.get(i));
                if (key.indexOf("@image") != -1){
                    int rowNum = r + 1; //excel表头是从第二行开始读，索引需要加1
                    String imageKey = rowNum + "," + i;
                    String imagePath = fileMap.get(imageKey);
                    if(StringUtils.isNotEmpty(imagePath)) {
                        value = imagePath;
                    }
                }
                formJson.put(key, value);
                for (int j = 0; j < formList.size(); j++) {
                    JSONObject jsonObject = formList.getJSONObject(j);
                    if(jsonObject.getString("_id").equals(String.valueOf(header.get(i)))){
                        jsonObject.put("value",value);
                        break;
                    }
                }
            }
            entity.setFormContent(formJson.toJSONString());
            entity.setFormTemplate(formContent.toJSONString());
            if(context!=null && context.getUser()!=null) {
                entity.setCreateUserId(context.getUser().getId());
            }
            entity.setCreateTime(new Date());
            applyLogList.add(entity);
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        for(ApplyLogEntity applyLogEntity:applyLogList){
            try {
                this.generateOriginal(applyLogEntity, context);
                successNum++;
            }catch (Exception var11) {
                this.log.error("导入异常", var11);
                ++failureNum;
            }
        }
        successMsg.insert(0, "数据导入成功！共 " + successNum + " 条");
        if (failureNum > 0) {
            failureMsg.insert(0, "导入失败！共 " + failureNum + " 条数据格式不正确");
            successMsg.append("\n");
            successMsg.append(failureMsg);
        }

        return successMsg.toString();
    }

    @Override
    public ApplyLogEntity getBySocialCode(String socialCode) throws AppException {
        ApplyLogEntity entity = this.selectOne(new ApplyLogQuery().socialCode(socialCode));
        if(entity!=null){
            List<ChildLicenseEntity> childs = childLicenseService.find(new ChildLicenseQuery().applyId(entity.getId()).processStatus(ProcessStatusEnum.已处理.getValue()).childStatus(StatusEnum.ENABLE.getValue()));
            entity.setChildLicense(childs);
        }
        return entity;
    }

}